(***********************************************************************)
(*                                                                     *)
(*                       Ambients in JoCaml                            *)
(*                                                                     *)
(*               Alan Schmitt, projet Para, INRIA Rocquencourt         *)
(*                                                                     *)
(*  Copyright 1999 Institut National de Recherche en Informatique et   *)
(*  Automatique.  Distributed only by permission.                      *)
(*                                                                     *)
(***********************************************************************)

open Syntax
open Lexer
open Print
open Trad

let parse_phrase aparser alexer lexbuf =
  let orig = lexbuf.Lexing.lex_abs_pos + lexbuf.Lexing.lex_curr_pos in
  let orig = if orig > 0 then orig+1 else orig in
  try
    let phrase = aparser alexer lexbuf in Parsing.clear_parser(); phrase
  with Parsing.Parse_error ->
         let pos1 = Lexing.lexeme_start lexbuf - orig in
         let pos2 = Lexing.lexeme_end lexbuf - orig in
           if Lexing.lexeme lexbuf <> ";;" then
           while alexer lexbuf <> Parser.SEMISEMI do () done;
           failwith
             ("Characters "^(string_of_int pos1)^"-"^
                      (string_of_int pos2)^":Syntax error")
    | Lexer.Illegal ->
         let pos1 = Lexing.lexeme_start lexbuf - orig in
         let pos2 = Lexing.lexeme_end lexbuf - orig in
           failwith
             ("Characters "^(string_of_int pos1)^"-"^
                      (string_of_int pos2)^":Lexical error "^
                      (Lexing.lexeme lexbuf))
;;
	   
let main () =

  let args = Sys.argv in
  let n = Array.length args in

  print_string "starting\n";
  flush stdout;
  let in_chan = ref stdin in
  let start_amb = ref "*toploc" in 
  for i = 1 to n/2 do
    if (args.(2*i-1) = "-file")
    then in_chan := open_in args.(2*i)
    else if (args.(2*i-1) = "-amb")
    then start_amb := args.(2*i)
  done;
  flush stdout;
  try
    let lexbuf = Lexing.from_channel !in_chan in
    while true do
      let d = parse_phrase Parser.main Lexer.token lexbuf in
      print_amb_expr d;
      print_newline();
      flush stdout;
      let g_id = Ns.lookup "get_id" vartype in
      let base_id = g_id () in
      print_string "my evaluation base_id is ";
      print_int base_id;
      print_newline ();
      let f = Ns.lookup !start_amb vartype in
      let f_to_be_used = {e = {there = top_location; sub = f.e.sub; sub_in = f.e.sub_in; sub_out = f.e.sub_out}; a_in = f.a_in; a_out = f.a_out; a_open = f.a_open; comm_in = f.comm_in; comm_out = f.comm_out} in
      print_string "starting translation\n";
      spawn {trad f_to_be_used [] None base_id d;}
    done
  with Eof -> ()
;;
      
Printexc.catch main ();
Join.server ();;
      
